//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
// ************************************************************
// setup.cpp
// 
// Implementation of DllMain and setup functions
//
//
// ************************************************************
#include "setup.h"

HINSTANCE g_hinstModule;


BOOL APIENTRY DllMain(
    HANDLE hModule, 
    DWORD  ul_reason_for_call, 
    LPVOID lpReserved
    )
{
    switch (ul_reason_for_call)
    {
        case DLL_PROCESS_ATTACH:
        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
        case DLL_PROCESS_DETACH:
            g_hinstModule = (HINSTANCE)hModule;
            break;
    }
    return TRUE;
}


// **************************************************************************
// Function Name: Install_Init
// 
// Purpose: processes the push message.
//
// Arguments:
//    IN HWND hwndParent  handle to the parent window
//    IN BOOL fFirstCall  indicates that this is the first time this function is being called
//    IN BOOL fPreviouslyInstalled  indicates that the current application is already installed
//    IN LPCTSTR pszInstallDir  name of the user-selected install directory of the application
//
// Return Values:
//    codeINSTALL_INIT
//    returns install status
//
// Description:  
//    The Install_Init function is called before installation begins.
//    User will be prompted to confirm installation.
// **************************************************************************
SETUP_API codeINSTALL_INIT Install_Init(
    HWND        hwndParent,
    BOOL        fFirstCall,     // is this the first time this function is being called?
    BOOL        fPreviouslyInstalled,
    LPCTSTR     pszInstallDir
    )
{
	wstring dir=pszInstallDir;
	wstring myResources=dir+L"\\resources\\VGA\\\0";
	//Delete dir :
	SHFILEOPSTRUCT dirToRemove;
	memset(&dirToRemove, 0, sizeof(dirToRemove));
	dirToRemove.wFunc=FO_DELETE;
	dirToRemove.pFrom=myResources.c_str();
	dirToRemove.fFlags=FOF_NOCONFIRMATION|FOF_SILENT;

	SHFileOperation(&dirToRemove);
	//ask for other shutdown prog suppression?
    return codeINSTALL_INIT_CONTINUE;
}


// **************************************************************************
// Function Name: Install_Exit
// 
// Purpose: processes the push message.
//
// Arguments:
//    IN HWND hwndParent  handle to the parent window
//    IN LPCTSTR pszInstallDir  name of the user-selected install directory of the application
//
// Return Values:
//    codeINSTALL_EXIT
//    returns install status
//
// Description:  
//    Register query client with the PushRouter as part of installation.
//    Only the first two parameters really count.
// **************************************************************************
SETUP_API codeINSTALL_EXIT Install_Exit(
    HWND    hwndParent,
    LPCTSTR pszInstallDir,      // final install directory
    WORD    cFailedDirs,
    WORD    cFailedFiles,
    WORD    cFailedRegKeys,
    WORD    cFailedRegVals,
    WORD    cFailedShortcuts
    )
{
	//change the previous link:
	MoveFile(L"\\Windows\\longpressendkey.lnk",L"\\Windows\\longpressendkey.old");
	MoveFile(L"\\Windows\\Long_Power.lnk",L"\\Windows\\Long_Power.old");
	MoveFile(L"\\Windows\\PowerOffWarning.exe",L"\\Windows\\PowerOffWarningOld.exe");

	HKEY myKey;
	DWORD disposition;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE\\Manila_QuickShutdown", 0, 0, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, 0, &myKey, &disposition);
	bool isNew=disposition==REG_CREATED_NEW_KEY;

	wstring dir=pszInstallDir;
	wstring myResources=dir+L"\\resources\\VGA\\\0";
	RegSetValueEx(myKey, L"SourcesDirectory", 0, REG_SZ, (const BYTE*)myResources.c_str(), myResources.size()*sizeof(WCHAR));//set the new info

	if(isNew){
		RegSetValueEx(myKey, L"langFile", 0, REG_SZ, (BYTE*)(myResources+L"lang.english.txt").c_str(), (myResources+L"lang.english.txt").length()*sizeof(WCHAR));

		unsigned char val=1;
		RegSetValueEx(myKey, L"ShutdownActiv", 0, REG_BINARY, (BYTE*)&val, sizeof(val));
		RegSetValueEx(myKey, L"RestartActiv", 0, REG_BINARY, (BYTE*)&val, sizeof(val));
		RegSetValueEx(myKey, L"SleepActiv", 0, REG_BINARY, (BYTE*)&val, sizeof(val));

		val=0;
		RegSetValueEx(myKey, L"ScreenOffActiv", 0, REG_BINARY, (BYTE*)&val, sizeof(val));
		RegSetValueEx(myKey, L"BootActiv", 0, REG_BINARY, (BYTE*)&val, sizeof(val));
		RegSetValueEx(myKey, L"ProgActiv", 0, REG_BINARY, (BYTE*)&val, sizeof(val));

	}
	RegCloseKey(myKey);

	//copy the new link:
	wstring myProg=L"\\QuickShutdown.exe";
	CopyFile((dir+myProg).c_str(),L"\\Windows\\PowerOffWarning.exe",false);

	myProg=dir+L"\\QuickShutdown.exe\"";
	WCHAR buffer[255];
	buffer[0]=L'"';
	// copier la chane
	wcsncpy( &buffer[1], myProg.c_str(), myProg.length()*sizeof(WCHAR) );
	SHCreateShortcut( L"\\Windows\\Long_Power.lnk",buffer);
	SHCreateShortcut( L"\\Windows\\longpressendkey.lnk",buffer);

	//Install Service:
	//create registry keys (example for telnet):
//	[HKEY_LOCAL_MACHINE\Services\TELNETD]
//	"Dll"="TELNETD.DLL"
//	"Order"=dword:8
//	"Keep"=dword:1
//	"Prefix"="TEL"
//	"Index"=dword:0
//	"Context"=dword:1
//	"Description"="Telnet Server"
//	"Description"="Services incoming telnet requests"
//	"Flags"="dword:0 -> or DEVFLAGS_LOADLIBRARY (2)
	

	RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"Services\\powerListener", 0, 0, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, 0, &myKey, &disposition);
	
	wstring valueString=L"HookPower.dll";
	RegSetValueEx(myKey, L"Dll", 0, REG_SZ, (const BYTE*)valueString.c_str(), valueString.size()*sizeof(WCHAR));//set the new info
	valueString=L"HPN";
	RegSetValueEx(myKey, L"Prefix", 0, REG_SZ, (const BYTE*)valueString.c_str(), valueString.size()*sizeof(WCHAR));//set the new info
	valueString=L"record power evolution";
	RegSetValueEx(myKey, L"DisplayName", 0, REG_SZ, (const BYTE*)valueString.c_str(), valueString.size()*sizeof(WCHAR));//set the new info
	DWORD number=0xE;//arbitrary high number (should be started last)
	RegSetValueEx(myKey, L"Order", 0, REG_DWORD, (const BYTE*)&number, sizeof(DWORD));//set the new info
	number=1;
	RegSetValueEx(myKey, L"Keep", 0, REG_DWORD, (const BYTE*)&number, sizeof(DWORD));//set the new info
	number=0;
	RegSetValueEx(myKey, L"Index", 0, REG_DWORD, (const BYTE*)&number, sizeof(DWORD));//set the new info
	number=0;
	RegSetValueEx(myKey, L"Context", 0, REG_DWORD, (const BYTE*)&number, sizeof(DWORD));//set the new info
	number=0;
	RegSetValueEx(myKey, L"Flags", 0, REG_DWORD, (const BYTE*)&number, sizeof(DWORD));//set the new info

	RegCloseKey(myKey);

	HANDLE hService;
	hService = RegisterService(L"HPN",0,L"HookPower.dll", 0);
	if((long)hService==ERROR_FILE_NOT_FOUND)
		MessageBox(NULL,L"Service run",L"ERROR can't find HookPower.dll",MB_OK|MB_ICONERROR);
	if((long)hService==ERROR_INVALID_FUNCTION)
		MessageBox(NULL,L"Service run",L"ERROR HookPower.dll have not been build correctly",MB_OK|MB_ICONERROR);

    return codeINSTALL_EXIT_DONE;
}


// **************************************************************************
// Function Name: Uninstall_Init
// 
// Purpose: processes the push message.
//
// Arguments:
//    IN HWND hwndParent  handle to the parent window
//    IN LPCTSTR pszInstallDir  name of the user-selected install directory of the application
//
// Return Values:
//    codeUNINSTALL_INIT
//    returns uninstall status
//
// Description:  
//    Query the device data using the query xml in the push message,
//    and send the query results back to the server.
// **************************************************************************
SETUP_API codeUNINSTALL_INIT Uninstall_Init(
    HWND        hwndParent,
    LPCTSTR     pszInstallDir
    )
{
	HKEY myKey;
	DWORD disposition=0;
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"Services", 0, 0, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, 0, &myKey, &disposition);
	RegDelnodeRecurse(myKey,L"powerListener");
	RegCloseKey(myKey);

	DeleteFile(L"\\Windows\\Long_Power.lnk");
	DeleteFile(L"\\Windows\\PowerOffWarning.exe");
	DeleteFile(L"\\Windows\\longpressendkey.lnk");
	//set the previous link:
	MoveFile(L"\\Windows\\Long_Power.old",L"\\Windows\\Long_Power.lnk");
	MoveFile(L"\\Windows\\longpressendkey.old",L"\\Windows\\longpressendkey.lnk");
	MoveFile(L"\\Windows\\PowerOffWarningOld.exe",L"\\Windows\\PowerOffWarning.exe");


	RegCreateKeyEx(HKEY_LOCAL_MACHINE, L"SOFTWARE", 0, 0, REG_OPTION_NON_VOLATILE,
		KEY_ALL_ACCESS, 0, &myKey, &disposition);

	RegDelnodeRecurse(myKey,L"Manila_QuickShutdown\\");
	RegCloseKey(myKey);

	HANDLE hService = GetServiceHandle(L"HPN0:", NULL, 0);
	DeregisterService(hService);

    return codeUNINSTALL_INIT_CONTINUE;
}


// **************************************************************************
// Function Name: Uninstall_Exit
// 
// Purpose: processes the push message.
//
// Arguments:
//    IN HWND hwndParent  handle to the parent window
//
// Return Values:
//    codeUNINSTALL_EXIT
//    returns uninstall status
//
// Description:  
//    Query the device data using the query xml in the push message,
//    and send the query results back to the server.
// **************************************************************************
SETUP_API codeUNINSTALL_EXIT Uninstall_Exit(
    HWND    hwndParent
    )
{
    return codeUNINSTALL_EXIT_DONE;
}
//*************************************************************
//
//  RegDelnodeRecurse()
//
//  Purpose:    Deletes a registry key and all it's subkeys / values.
//
//  Parameters: hKeyRoot    -   Root key
//              lpSubKey    -   SubKey to delete
//
//  Return:     TRUE if successful.
//              FALSE if an error occurs.
//
//*************************************************************

BOOL RegDelnodeRecurse (HKEY hKeyRoot, LPTSTR lpSubKey)
{
    LPTSTR lpEnd;
    LONG lResult;
    DWORD dwSize;
    TCHAR szName[MAX_PATH];
    HKEY hKey;
    FILETIME ftWrite;

    // First, see if we can delete the key without having
    // to recurse.

    lResult = RegDeleteKey(hKeyRoot, lpSubKey);

    if (lResult == ERROR_SUCCESS) 
        return TRUE;

    lResult = RegOpenKeyEx (hKeyRoot, lpSubKey, 0, KEY_READ, &hKey);

    if (lResult != ERROR_SUCCESS) 
    {
		return FALSE;
    }

    // Check for an ending slash and add one if it is missing.

    lpEnd = lpSubKey + lstrlen(lpSubKey);

    if (*(lpEnd - 1) != TEXT('\\')) 
    {
        *lpEnd =  TEXT('\\');
        lpEnd++;
        *lpEnd =  TEXT('\0');
    }

    // Enumerate the keys

    dwSize = MAX_PATH;
    lResult = RegEnumKeyEx(hKey, 0, szName, &dwSize, NULL,
                           NULL, NULL, &ftWrite);

    if (lResult == ERROR_SUCCESS) 
    {
        do {

            StringCchCopy (lpEnd, MAX_PATH*2, szName);

            if (!RegDelnodeRecurse(hKeyRoot, lpSubKey)) {
                break;
            }

            dwSize = MAX_PATH;

            lResult = RegEnumKeyEx(hKey, 0, szName, &dwSize, NULL,
                                   NULL, NULL, &ftWrite);

        } while (lResult == ERROR_SUCCESS);
    }

    lpEnd--;
    *lpEnd = TEXT('\0');

    RegCloseKey (hKey);

    // Try again to delete the key.

    lResult = RegDeleteKey(hKeyRoot, lpSubKey);

    if (lResult == ERROR_SUCCESS) 
        return TRUE;

    return FALSE;
}